/*
editeurconfig - Editeur de Fichiers de Config de Jaxe

Copyright (C) 2011 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package editeurconfig;

import jaxe.InterfaceSchema;
import jaxe.wxs.JaxeWXS;
import jaxe.SchemaRelaxNG;
import jaxe.SchemaSimple;
import jaxe.Config;
import jaxe.JaxeException;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.border.*;
import java.util.*;
import java.net.URL;
import org.w3c.dom.*;
import java.io.*;


/**
* Reprsente la liste des lments
*/
public class ListeElements {
    
    private InterfaceSchema schema;
    
    private ArrayList<Element> tabRefEl = new ArrayList<Element>();
    
    private JTextField jTexteRecherche;	
    private ArrayList<String> lesElements = new ArrayList<String>();
    private ArrayList<String> nomsElementsRacinesP = new ArrayList<String>();
    private JList listeDesElements;
    private DefaultListModel listModel;
    //private DefaultListModel nouveauModele;
    
    private JList listeDesElementsRacinesP;
    private DefaultListModel modelRacinesP;
    
    private ArrayList<String> listeAttributs = new ArrayList<String>();
    
    /**
    * Rcupre les lments d'un schma  partir de son url
    */
    public ListeElements(final URL schemaURL, final Config cfg, final Element schema_simple) {
        if (schemaURL != null && schema_simple == null) {
            if (schemaURL.toString().endsWith("xsd") || schemaURL.toString().endsWith("XSD")) {
                try {
                    schema = new JaxeWXS(schemaURL, cfg);
                } catch (final JaxeException jex) {
                    System.out.println("JaxeWXS: "+jex);
                } 
            }
            
            if (schemaURL.toString().endsWith("rng") || schemaURL.toString().endsWith("RNG")) {
                schema = new SchemaRelaxNG(schemaURL, cfg);
            }
        }
        
        if (schema_simple != null && schemaURL == null)
            schema = new SchemaSimple(schema_simple, cfg);
        
        if (schema != null) {
            tabRefEl = schema.listeTousElements();
            recupererLesElements();
        }
    }
    
    
    /**
    * Rcupre la liste des noms des lments du schma, et les trie par nom croissant
    */
    public void recupererLesElements() {
        lesElements.clear();   
        
        for (int i = 0; i< tabRefEl.size();i++) {
            String nomEl = schema.nomElement(tabRefEl.get(i));
            if (!lesElements.contains(nomEl))
                lesElements.add(nomEl);
        }
        
        Collections.sort(lesElements);
    }
    
    /**
    * Renvoie la liste des noms des lments
    * @return La liste des noms des lments
    */
    public ArrayList<String> getLesElements() {
        return lesElements;
    }
    
    /**
    * Renvoie la liste des rfrences de tous les lments
    * @return la liste des rfrences de tous les lments
    */
    public ArrayList<Element> getReferencesElements() {
        Collections.sort(tabRefEl, new ComparateurElement(schema));
        return tabRefEl;
    }
    
    /**
    * Retourne la liste des attributs d'un lment du schma
    * @param La rference de l'lment
    * @return La liste des attributs de l'lment el
    */
    public ArrayList<String> getAttributElement(final Element el) {
        listeAttributs.clear();
        if (schema != null) {
            ArrayList<Element> lAtt = schema.listeAttributs(el);
            if (lAtt != null && lAtt.size() > 0) {
                for (int i=0; i<lAtt.size();i++)
                    listeAttributs.add(schema.nomAttribut(lAtt.get(i)));
                return listeAttributs;
            }
        }
        return null;
    }
    
    
    /**
    * Retourne la rference d'un lment dans le schma  partir du nom
    * @param Le nom de l'lment sous forme de String
    * @return La rference de l'lment
    */
    public Element getReferenceElement(String s) {
        if (schema != null)
            return schema.referenceElement(s);
        return null;
    }
    
    
    /**
    * Retourne la liste des noms des lments racines possibles du schma classs par ordre alphabtique
    */
    public ArrayList<String> getNomsElementsRacinesP(final boolean meilleureRacine) {
        final ArrayList<Element> listeElementsRacinesP = schema.listeElementsRacines();
        for (int i = 0; i< listeElementsRacinesP.size();i++) {
            ArrayList<Element> listeDesParents = new ArrayList<Element>();
            listeDesParents = schema.listeElementsParents(listeElementsRacinesP.get(i));
            if (!nomsElementsRacinesP.contains(schema.nomElement(listeElementsRacinesP.get(i)))) {
                if (meilleureRacine) {
                    if (listeDesParents.size() == 0)
                        nomsElementsRacinesP.add(schema.nomElement(listeElementsRacinesP.get(i)));
                    else {
                        if (listeDesParents.size() == 1 && listeDesParents.contains(listeElementsRacinesP.get(i)))
                            nomsElementsRacinesP.add(schema.nomElement(listeElementsRacinesP.get(i)));
                        /*else if (elementDeBloc(listeElementsRacinesP.get(i)))
                            nomsElementsRacinesP.add(schema.nomElement(listeElementsRacinesP.get(i)));*/
                    }
                }
                else
                    nomsElementsRacinesP.add(schema.nomElement(listeElementsRacinesP.get(i)));
            }
        }
        Collections.sort(nomsElementsRacinesP);
        return nomsElementsRacinesP;
    }
    
    public JList JListeElementsRacinesP(final boolean meilleureRacine) {
        getNomsElementsRacinesP(meilleureRacine);
        if (listeDesElementsRacinesP == null) {
            listeDesElementsRacinesP = new JList();
            listeDesElementsRacinesP.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            modelRacinesP = new DefaultListModel();
            for (int i = 0; i< nomsElementsRacinesP.size();i++)
                modelRacinesP.addElement(nomsElementsRacinesP.get(i));
        }
        listeDesElementsRacinesP.setModel(modelRacinesP);
        
		return listeDesElementsRacinesP;
    }
    
    
    
    /**
    * Retourne la liste de tous les lments du schma
    * @return La liste de tous les lments dans une JList
    */
    public JList JListeDesElements() {
		if (listeDesElements == null) {
			listeDesElements = new JList();
            listeDesElements.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			listModel = new DefaultListModel();
			for (int i = 0; i< lesElements.size();i++)
                listModel.addElement(lesElements.get(i));
			listeDesElements.setModel(listModel);
		}
		return listeDesElements;
    }
    
    
    /**
    * Retourne un champ texte qui permet de filtrer une JList
    * @return Un champ JTextField
    */
    public JTextField getJTexteRecherche() {
        
        if (jTexteRecherche == null) {
            jTexteRecherche = new JTextField(15);
            jTexteRecherche.setMaximumSize(jTexteRecherche.getPreferredSize());
			//Ajout d'un listener lors d'un relachement d'une touche de clavier
			jTexteRecherche.addKeyListener(new KeyAdapter() {
				public void keyReleased(KeyEvent e) {
                    DefaultListModel nouveauModele = new DefaultListModel();
					String texteSaisi = jTexteRecherche.getText();					
					for (int i = 0; i < lesElements.size(); i++) {
						//afficher que les elements commencants par le texte saisi
						if (lesElements.get(i).toLowerCase().startsWith(texteSaisi.toLowerCase())) {
							nouveauModele.addElement(lesElements.get(i));
						}
					}
					listeDesElements.setModel(nouveauModele);
				}
			});				
		}
		return jTexteRecherche;
	}
    
    /**
    * Retourne le modle de la liste
    * @return Le modle de la JList
    */
    public DefaultListModel getListModel() {
        return listModel;
    }
    
    /**
    * Retourne le modle de la liste des lments Racines possibles
    * @return Le modle de la JList
    */
    public DefaultListModel getListModelRP() {
        return modelRacinesP;
    }
    
    
    
    /**
    * Renvoie true si le parent de l'lment dont on donne la rfrence peut contenir du texte
    */
    public boolean parentContientDuTexte(final Element refElement) {
        boolean b = false;
        ArrayList<Element> listeElementsParents = new ArrayList<Element>();
        listeElementsParents = schema.listeElementsParents(refElement);
        for (int i = 0; i < listeElementsParents.size(); i++) {
            if (schema.contientDuTexte(listeElementsParents.get(i))) {
                b = true;
                break;
            }
        }
        return b;
    }
    
    
    /**
    * Renvoie true si l'lment dont on donne la rfrence n'est pas mlang  du texte
    */
    public boolean elementDeBloc(final Element refElement) {
        if (!parentContientDuTexte(refElement) /*&& (schema.contientDuTexte(refElement) || schema.listeSousElements(refElement).size() > 0)*/)
            return true;
        else
            return false;
    }
    
    /**
    * Renvoie true si l'lment dont on donne la rfrence peut tre mlang avec du texte
    */
    public boolean elementMelangeTexte(final Element refElement) {
        if (parentContientDuTexte(refElement) /*&& (schema.contientDuTexte(refElement) || schema.listeSousElements(refElement).size() > 0)*/)
            return true;
        else
            return false;
    }
    
    
    /**
    * Retourne l'interface schema
    * @return L'interface schema
    */
    public InterfaceSchema getInterfaceSchema() {
        return schema;
    }
}



/**
* Classe permet de trier les lments de schma par ordre alphabtique
*/
class ComparateurElement implements Comparator<Element> {
    InterfaceSchema schema;
    public ComparateurElement(final InterfaceSchema schema) {
        this.schema = schema;
    }
    
    public int compare(Element el1, Element el2) {
        if (schema.nomElement(el1).compareTo(schema.nomElement(el2)) < 0)
            return -1;
        else if (schema.nomElement(el1).compareTo(schema.nomElement(el2)) == 0)
            return 0;        	
        else
            return 1;
    }
}
